/*!
 * provision
 * Copyright(c) 2011 Simyung Kim
 * MIT Licensed
 */

var provision = module.exports = {
  // useragent, monomi, geoip
  mod: {},

  // configurable functions
  _steps: {},

  // express middleware
  middleware: function(options) {
    var self = this;
    self.options = options || {};
    
    if (!self.options.monomi) self.options.monomi = true;
    if (!self.options.useragent) self.options.useragent = true;
    if (!self.options.language) self.options.language = true;
    if (!self.options.geoip) self.options.geoip = true;

    return function(req, res, next) {
      send(req, res, next);
    };
  },
  canProvisioning: function(fn) {
    if (typeof fn === 'function') {
      this._steps.canProvisioning = fn;
    }
    return this;
  },
  afterProvisioning: function(fn) {
    if (typeof fn === 'function') {
      this._steps.afterProvisioning = fn;
    }
    return this;
  }
};

// setup default confiruable functions
provision
  .canProvisioning(function(req, options) {
    if (!options.monomi && !options.geoip && !options.language && !options.useragent)
      return false;
    if (req.session && req.session.provision)
      return false;
    return true;
  })
  .afterProvisioning(function(req, res, next, data) {
    var prov = {};
    prov.locale = data.language;
    prov.country = data.country;
    prov.family = data.family;
    prov.os = data.os;
    prov.osVersion = data.osVersion;
    prov.browserType = data.browserType;

    if (data.location) {
      prov.location = {};
      prov.location.latitude = data.location.latitude;
      prov.location.longitude = data.location.longitude;
    }
    
    if (req.session) {
      req.session.provision = prov;
    }
    else {
      req.provision = prov;
    }
    next();
  });

var send = exports.send = function(req, res, next){
  var self = provision;
  if (!self._steps.canProvisioning(req, self.options)) {
    return next();
  }

  // create provision information when session creation time.
  var prov = {};
  
  if (self.options.geoip) {
    findGeoLocation(req, prov);
  }

  if (self.options.language || self.options.geoip) {
    guessLanguage(req, prov);
  }

  if (self.options.monomi) {
    detectBrowserType(req, prov);
  }

  if (self.options.useragent) {
    parseUseragent(req, prov);
  }

//  console.log(prov);

  self._steps.afterProvisioning(req, res, next, prov);
};

function getRemoteAddress(req) {
  var ip_address = null;
  try {
    ip_address = req.headers['x-forwarded-for'];
    if (ip_address === undefined) {
      ip_address = req.connection.remoteAddress;
    }
  }
  catch ( error ) {
    ip_address = req.connection.remoteAddress;
  }
  
  var remoteAddress;
  if(ip_address === "127.0.0.1") {
    remoteAddress = "8.8.8.8";
  } else {
    remoteAddress = ip_address;
  }

  return remoteAddress;
}

function findGeoLocation(req, prov) {
  var self = provision;
  if (!self.mod['geoip']) {
    var geoip = require('geoip');

    var cityPath = process.cwd() + '/GeoLiteCity.dat';
    try {
      self.cities = new geoip.City(cityPath);
      self.mod['geoip'] = geoip;
    } catch(e) {
      console.log("Couldn't load geoip database. " + cityPath);
      return;
    }
  }

  location = self.cities.lookupSync(getRemoteAddress(req));
  if (location) {
    prov.location = location;
    prov.country = location.country_code.toLowerCase();
  }
//  console.log(location);
}

//guess language setting based on http headers
function guessLanguage(req, prov){
  if(typeof req === 'object'){
    var language_header = req.headers['accept-language'],
    languages = [];
    regions = [];

    if (language_header) {
        language_header.split(',').forEach(function(l) {
            header = l.split(';', 1)[0];
            lr = header.split('-', 2);
            if (lr[0]) {
                languages.push(lr[0].toLowerCase());
            }
            if (lr[1]) {
                regions.push(lr[1].toLowerCase());
            }
        });
    }
    prov.language = languages.length > 0 ? languages[0] : 'en';
    if (!prov.country) prov.country = regions.length > 0 ? regions[0] : 'us';
  }
}

function detectBrowserType(req, prov) {
  var self = provision;
  if (!self.mod['monomi']) {
    self.mod['monomi'] = require('monomi');
  }
  var monomiMw = self.mod.monomi.detectBrowserType();
  monomiMw(req, null, function(){});
  prov.browserType = req.monomi.browserType;
}

function parseUseragent(req, prov) {
  var self = provision;
  if (!self.mod['useragent']) {
    self.mod['useragent'] = require('useragent');
  }
  var agent = self.mod.useragent.parse(req.headers['user-agent']);
  prov.family = agent.family;
  prov.os = agent.os;
  prov.osVersion = agent.major;
}
